#ifndef _MT2060_STRUCT_H
#define _MT2060_STRUCT_H

/*
**  Two-wire serial bus subaddresses of the tuner registers.
**  Also known as the tuner's register addresses.
*/

static enum MT2060_Register_Offsets
{
	PART_REV = 0,      //  0x00: Part/Rev Code        
	LO1C_1,         //  0x01: LO1C Byte 1          
	LO1C_2,         //  0x02: LO1C Byte 2          
	LO2C_1,         //  0x03: LO2C Byte 1          
	LO2C_2,         //  0x04: LO2C Byte 2          
	LO2C_3,         //  0x05: LO2C Byte 3          
	LO_STATUS,      //  0x06: LO Status Byte       
	FM_FREQ,        //  0x07: FM FREQ Byte         
	MISC_STATUS,    //  0x08: Misc Status Byte     
	MISC_CTRL_1,    //  0x09: Misc Ctrl Byte 1     
	MISC_CTRL_2,    //  0x0A: Misc Ctrl Byte 2     
	MISC_CTRL_3,    //  0x0B: Misc Ctrl Byte 3     
	LO1_1,          //  0x0C: LO1 Byte 1           
	LO1_2,          //  0x0D: LO1 Byte 2           
	END_REGS
};


typedef unsigned char   U8Data;         /*  type corresponds to 8 bits      */
typedef unsigned int    UData_t;        /*  type must be at least 32 bits   */
typedef int             SData_t;        /*  type must be at least 32 bits   */
typedef void *          Handle_t;       /*  memory pointer type             */

#define MAX_ZONES                  48
#define MT2060_CNT                 (1)                

struct MT_ExclZone_t
{
    UData_t         min_;
    UData_t         max_;
    struct MT_ExclZone_t*  next_;
};

/*
**  Structure of data needed for Spur Avoidance
*/
typedef struct
{
    UData_t nAS_Algorithm;
    UData_t f_ref;
    UData_t f_in;
    UData_t f_LO1;
    UData_t f_if1_Center;
    UData_t f_if1_Request;
    UData_t f_if1_bw;
    UData_t f_LO2;
    UData_t f_out;
    UData_t f_out_bw;
    UData_t f_LO1_Step;
    UData_t f_LO2_Step;
    UData_t f_LO1_FracN_Avoid;
    UData_t f_LO2_FracN_Avoid;
    UData_t f_zif_bw;
    UData_t f_min_LO_Separation;
    UData_t maxH1;
    UData_t maxH2;
    UData_t bSpurPresent;
    UData_t bSpurAvoided;
    UData_t nSpursFound;
    UData_t nZones;
    struct MT_ExclZone_t* freeZones;
    struct MT_ExclZone_t* usedZones;
    struct MT_ExclZone_t MT_ExclZones[MAX_ZONES];
} MT_AvoidSpursData_t;

typedef struct
{
    Handle_t            handle;
    Handle_t            hUserData;
    UData_t             address;
    UData_t             version;
    UData_t             tuner_id;
    MT_AvoidSpursData_t AS_Data;
    UData_t             f_IF1_actual;
    UData_t             num_regs;
    U8Data              reg[END_REGS];
    UData_t             RF_Bands[10];
}  MT2060_Info_t;

typedef struct TUNER_MT2060 {
	MT2060_Info_t MT2060_Info[MT2060_CNT];
	Handle_t      MT2060_handle;
	UData_t       MT2060_FMF;
} TUNER_MT2060;

#endif

