

#include "mt2060_basic.h"
#include "MT2060Tuner.h"
#include "MT2060Struct.h"
#include "rtd2830_priv.h"




int
mt2060_tuner_init(
    struct dvb_frontend*	fe)
{

	struct rtd2830_state*	p_state= (struct rtd2830_state*)fe->demodulator_priv;
	TUNER_MT2060*		mt2060_tuner = (TUNER_MT2060 *)(p_state->pTuner);
	UData_t			MT2060_Addr =  p_state->tuner_addr;
	
	Handle_t		hMT2060 = NULL;
	UData_t			MTStatus = MT_OK;
	UData_t			f_IF1 = mt2060_tuner->MT2060_FMF;

	printk(("CDevice::MT2060TunerInit -- begin\n"));
		  
	MTStatus = MT2060_Open(MT2060_Addr, &hMT2060, (Handle_t)fe);
	if(!MT_NO_ERROR(MTStatus))
	{
		printk(("CDevice::MT2060TunerInit -- Open Tuner Error!! \n"));
		return 1;
	}	

	//perform tuner driver overrides here
	MTStatus |= MT2060_SetParam(hMT2060, MT2060_IF1_CENTER, f_IF1);
	if(!MT_NO_ERROR(MTStatus))
	{
		printk(("CDevice::MT2060TunerInit -- MT2060_SetParam Error!! \n"));
		return 1;
	}
	
	MTStatus |= MT2060_SetGainRange(hMT2060, MT2060_VGAG_5DB);
	if(!MT_NO_ERROR(MTStatus))
	{
		printk(("CDevice::MT2060TunerInit -- MT2060_SetGainRange!! \n"));
		return 1;
	}  

	mt2060_tuner->MT2060_handle = hMT2060;

	printk(("CDevice::MT2060TunerInit -- end\n"));

	return 0;	
}

int
set_mt2060_tuner(
    struct dvb_frontend*			fe,
    struct dvb_frontend_parameters*	fep)
{

//	Handle_t		h;
	UData_t		f_in;		//RF input center frequency   
	UData_t		f_IF1;	// desired 1st IF frequency    
	UData_t		f_out;	// IF output center frequency  
	UData_t		f_IFBW;	// IF output bandwidth + guard 	
	ULONG           bandwidth; 	

	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	TUNER_MT2060*		mt2060_tuner = (TUNER_MT2060 *)(p_state->pTuner);
	Handle_t		hMT2060 = mt2060_tuner->MT2060_handle;
	struct dvb_ofdm_parameters*	p_ofdm_param= &fep->u.ofdm;	

	UData_t success = MT_OK;

	printk(("CDevice::SetMT2060Tuner -- begin\n"));
	
	f_in = fep->frequency;
	f_IF1 = mt2060_tuner->MT2060_FMF; //IF1_CENTER;
	f_out = 36125000UL;

	switch (p_ofdm_param->bandwidth) 
	{
	case BANDWIDTH_6_MHZ:		   
		bandwidth = 6;
		break;
		
	case BANDWIDTH_7_MHZ:
		bandwidth = 7;
		break;
		
	case BANDWIDTH_8_MHZ:
	default:
		bandwidth = 8;
		break;
	}
	
	f_IFBW = bandwidth*1000000UL + 750000;

	//printk(("f_in = %d, f_IF1 = %d, f_out = %d, f_IFBW = %d", f_in, f_IF1, f_out, f_IFBW));

	if (hMT2060 == NULL)
	{
		printk(("****Error hMT2060  Handle \n"));
		return 1;
	}  	
		
	success = MT2060_ChangeFreq(hMT2060,
		f_in,     // RF input center frequency  
		f_IF1,    // desired 1st IF frequency    
		f_out,    // IF output center frequency  
		f_IFBW
		);   /* IF output bandwidth + guard */
	if(!MT_NO_ERROR(success))
	{
		printk(("MT2060_ChangeFreq Fail!! \n"));
		return 1;
	}
	else
	{
		printk(("CDevice::SetMT2060Tuner -- end\n"));
	}

	return 0;	
}

int
mt2060_tuner_struct_init(
    struct dvb_frontend*			fe)
{
	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	TUNER_MT2060*	mt2060_tuner = (TUNER_MT2060 *)(p_state->pTuner);

	mt2060_tuner->MT2060_handle = NULL;	
	mt2060_tuner->MT2060_FMF = 1220000000;
	
	return 0;
}

