

#if !defined( __MT2060_H )
#define __MT2060_H

#include "mt_spuravoid.h"

#if defined( __cplusplus )     
extern "C"                     /* Use "C" external linkage                  */
{
#endif

//#define IF1_CENTER      (1220000000UL)  /* Center of the IF1 filter (in Hz) */

/*
**  Parameter for function MT2060_SetGPO that specifies the GPO pin
**  to be read/written.
*/
typedef enum
{
    MT2060_GPO_0 = 0x02,                /*  bitmask for GPO_0               */
    MT2060_GPO_1 = 0x04,                /*  bitmask for GPO_1               */
    MT2060_GPO   = 0x06,                /*  bitmask for GPO_0 | GPO_1       */
} MT2060_GPO_Mask;


/*
**  Values returned by the MT2060's on-chip temperature sensor
**  to be read/written.  Note the 40C resolution.
*/
typedef enum 
{
    MT2060_T_0C = 0,                    /*  Temperature < 0C                */
    MT2060_0C_T_40C,                    /*  0C < Temperature < 40C          */
    MT2060_40C_T_80C,                   /*  40C < Temperature < 80C         */
    MT2060_80C_T,                       /*  80C < Temperature               */
} MT2060_Temperature;


/*
**  Parameter for function MT2060_SetGainRange.
**  Note that the delta between min/max is the same, but the max
**  attainable gain is increased with non-default settings
**
**  MT2060_VGAG_0DB is the power-on default, but MT2060_VGAG_5DB is the
**  default value after initialization.
*/
typedef enum 
{
    MT2060_VGAG_0DB = 0,                /*  VGA gain range                  */
    MT2060_VGAG_5DB = 1,                /*  VGA gain range + 5 dB           */
    MT2060_VGAG_11DB = 3,               /*  VGA gain range + 11 dB          */
} MT2060_VGA_Gain_Range;


/*
**  Parameter for function MT2060_SetExtSRO that specifies the external
**  SRO drive frequency.
**
**  MT2060_EXT_SRO_OFF is the power-up default value.
*/
typedef enum
{
    MT2060_EXT_SRO_OFF,                 /*  External SRO drive off          */
    MT2060_EXT_SRO_BY_4,                /*  External SRO drive divide by 4  */
    MT2060_EXT_SRO_BY_2,                /*  External SRO drive divide by 2  */
    MT2060_EXT_SRO_BY_1                 /*  External SRO drive divide by 1  */
} MT2060_Ext_SRO;

/*
**  Parameter for function MT2060_GetParam & MT2060_SetParam that 
**  specifies the tuning algorithm parameter to be read/written.
*/
typedef enum
{
    /*  tuner address                                  set by MT2060_Open() */
    MT2060_IC_ADDR,

    /*  max number of MT2060 tuners       set by MT2060_CNT in mt_userdef.h */
    MT2060_MAX_OPEN,

    /*  current number of open MT2060 tuners           set by MT2060_Open() */
    MT2060_NUM_OPEN,
    
    /*  crystal frequency                            (default: 16000000 Hz) */
    MT2060_SRO_FREQ,

    /*  min tuning step size                            (default: 50000 Hz) */
    MT2060_STEPSIZE,

    /*  input center frequency                   set by MT2060_ChangeFreq() */
    MT2060_INPUT_FREQ,

    /*  LO1 Frequency                            set by MT2060_ChangeFreq() */
    MT2060_LO1_FREQ,
              
    /*  LO1 minimum step size                          (default: 250000 Hz) */
    MT2060_LO1_STEPSIZE,
    
    /*  LO1 FracN keep-out region                     (default: 2999999 Hz) */
    MT2060_LO1_FRACN_AVOID,

    /*  Current 1st IF in use                    set by MT2060_ChangeFreq() */
    MT2060_IF1_ACTUAL,
          
    /*  Requested 1st IF                         set by MT2060_ChangeFreq() */
    MT2060_IF1_REQUEST,
                
    /*  Center of 1st IF SAW filter                     located by customer */
    MT2060_IF1_CENTER,
                    
    /*  Bandwidth of 1st IF SAW filter               (default: 22000000 Hz) */
    MT2060_IF1_BW,
                    
    /*  zero-IF bandwidth                             (default: 3000000 Hz) */
    MT2060_ZIF_BW,

    /*  LO2 Frequency                            set by MT2060_ChangeFreq() */
    MT2060_LO2_FREQ,

    /*  LO2 minimum step size                           (default: 50000 Hz) */
    MT2060_LO2_STEPSIZE,

    /*  LO2 FracN keep-out region                       (default: 99999 Hz) */
    MT2060_LO2_FRACN_AVOID,

    /*  output center frequency                  set by MT2060_ChangeFreq() */
    MT2060_OUTPUT_FREQ,

    /*  output bandwidth                         set by MT2060_ChangeFreq() */
    MT2060_OUTPUT_BW,

    /*  min inter-tuner LO separation                 (default: 1000000 Hz) */
    MT2060_LO_SEPARATION,

    /*  ID of avoid-spurs algorithm in use            compile-time constant */
    MT2060_AS_ALG,

    /*  max # of intra-tuner harmonics                        (default: 5)  */
    MT2060_MAX_HARM1,

    /*  max # of inter-tuner harmonics                        (default: 5)  */
    MT2060_MAX_HARM2,

    /*  # of 1st IF exclusion zones used         set by MT2060_ChangeFreq() */
    MT2060_EXCL_ZONES,
                 
    /*  # of spurs found/avoided                 set by MT2060_ChangeFreq() */
    MT2060_NUM_SPURS,
          
    /*  >0 spurs avoided                         set by MT2060_ChangeFreq() */
    MT2060_SPUR_AVOIDED,
              
    /*  >0 spurs in output (mathematically)      set by MT2060_ChangeFreq() */
    MT2060_SPUR_PRESENT,

    MT2060_EOP                    /*  last entry in enumerated list         */
} MT2060_Param;

//  Description:    Initialize the tuner's register values.

UData_t MT2060_Open(UData_t MT2060_Addr,
                    Handle_t* hMT2060,
                    Handle_t hUserData);



//  Description:    Release the handle to the tuner.
UData_t MT2060_Close(Handle_t hMT2060);

//  Description:    Change the tuner's tuned frequency to f_in.
UData_t MT2060_ChangeFreq(Handle_t h,
                          UData_t f_in, 
                          UData_t f_IF1, 
                          UData_t f_out, 
                          UData_t f_IFBW);

//  Description:    Checks to see if LO1 and LO2 are locked.
UData_t MT2060_GetLocked(Handle_t h);

//  Description:    Initialize the tuner's register values.
UData_t MT2060_ReInit(Handle_t h);
 
UData_t MT2060_SetGainRange(Handle_t h, UData_t range);

UData_t MT2060_SetParam(Handle_t     h,
                        MT2060_Param param,
                        UData_t      nValue);

UData_t MT2060_LocateIF1(Handle_t h, 
								UData_t f_in, 
								UData_t f_out, 
								UData_t* f_IF1, 
								SData_t* P_max);

UData_t MT2060_GetFMF(Handle_t h);

#if defined( __cplusplus )
}
#endif

#endif

