
#include "mt_userdef.h"
#include "dvb_frontend.h"
#include "rtd2830_priv.h"
#include "tuner_demod_io.h"

//2Description:    Write values to device using a two-wire serial bus.

UData_t MT_WriteSub(
			Handle_t hUserData, 
			UData_t addr, 
			U8Data subAddress, 
			U8Data *pData, 
			UData_t cnt
						)
{
	UData_t					index_byte; 
	UData_t					status = MT_OK;                  
	struct dvb_frontend*		fe = (struct dvb_frontend*)hUserData; 	
	struct rtd2830_state*      	p_state= (struct rtd2830_state*)fe->demodulator_priv;
	struct dvb_usb_device*	dib = p_state->dib;	
	
	for( index_byte  = 0 ; index_byte < cnt ; index_byte++ )
		TUNER_WI2C( dib , addr , subAddress + index_byte , &pData[index_byte]  , 1 );
	

	return status;  
}


//2//2  Description:    Read values from device using a two-wire serial bus.

UData_t MT_ReadSub(
					Handle_t hUserData, 
					UData_t addr, 
					U8Data subAddress, 
					U8Data *pData, 
					UData_t cnt
					)
{
 	UData_t					index_byte; 	
	UData_t					status = MT_OK;                        /* Status to be returned        */
	struct dvb_frontend*		fe = (struct dvb_frontend*)hUserData;
	struct rtd2830_state*      	p_state= (struct rtd2830_state*)fe->demodulator_priv;
	struct dvb_usb_device*	dib = p_state->dib;	

	for( index_byte  = 0 ; index_byte < cnt ; index_byte++ )
		TUNER_RI2C( dib, addr, subAddress + index_byte, &pData[index_byte] , 1 );	
	

	return status;  
}


//2Description:    Delay execution for "nMinDelayTime" milliseconds

void MT_Sleep(
				Handle_t hUserData, 
				UData_t nMinDelayTime)
{
	UINT usec;
	do {
		usec = (nMinDelayTime > 8000) ? 8000 : nMinDelayTime;
		msleep(usec);
		nMinDelayTime -= usec;
		} while (nMinDelayTime > 0);
}



#if defined(MT2060_CNT)
#if MT2060_CNT > 0

//2//2  Description:    Measure the relative tuner gain using the demodulator

UData_t MT_TunerGain(
					Handle_t hUserData,
					SData_t* pMeas)
{
	UData_t 					status = MT_OK;                        /* Status to be returned        */
	struct dvb_frontend*		fe = (struct dvb_frontend*)hUserData;
	struct rtd2830_state*      	p_state= (struct rtd2830_state*)fe->demodulator_priv;
	struct dvb_usb_device*	dib = p_state->dib;	
	
	
	//2ToDo:  Add code here to return the gain / power level measured at the input to the demodulator.
	u8  data[2] = { 0 , 0 };	
	USHORT agc_lock=0;
	ULONG length =1 ;
	ULONG  IF_AGC_l=0;
	ULONG  IF_AGC_f=0;
	USHORT num=0x00;
	
	do{  
		//Read Demod agc_lock
		agc_lock=0;
		length=1;
		RTD2831_READ_BYTES( dib , RTD2830_PAGE_3 , 0x50 , &data[0] , length );
		
		
		agc_lock=data[0]&0x01;
		printk("agc_lock =%d\n",agc_lock);
		if (agc_lock)
		{
			//Read Demod IF_AGC
			length=2;
			RTD2831_READ_BYTES( dib , RTD2830_PAGE_3 , 0x59 , data , length);

			IF_AGC_l=(ULONG)(data[0]<<8)+data[1]; //chialing  070706
								
			if ((IF_AGC_l>=8192))
				IF_AGC_l=IF_AGC_l-8192;
			else
				IF_AGC_l=IF_AGC_l+8192;
						
			IF_AGC_f= (ULONG)((33000*IF_AGC_l)/(8192*2));
			printk("IF_AGC_l =%d\n",agc_lock);
			num=500;
		}
		
		num++;
		printk("num =%d agc_lock=%d\n",num,agc_lock);
			
	}while(num<500);
	
	
	*pMeas=(SData_t)(20*(33000-IF_AGC_f));   
	return (status);
}
#endif
#endif
