/*
 *  Driver for Realtek DVB-T rtd2830 demodulator
 */


#include <linux/time.h>

#include "rtd2831u.h"
#include "rtd2830_priv.h"
#include "tuner_demod_io.h"
#include "tuner.h"
#include "demod_rtl2830.h"

#define UPDATE_PROCEDURE_PERIOD	500   //500ms
#define TRUE		1
#define FALSE		0

#define RTD2830_DEBUG


#define MIN(a, b) ((a)< (b)? (a): (b))

static struct rtd2830_reg_addr rtd2830_reg_map[]= {
	/* RTD2831_RMAP_INDEX_USB_STAT         */		{ RTD2831U_USB, USB_STAT, 0, 7    },
	/*RTD2831_RMAP_INDEX_USB_EPA_CTL     */		{ RTD2831U_USB, USB_EPA_CTL, 0, 31    },
	/*RTD2831_RMAP_INDEX_USB_SYSCTL       */		{ RTD2831U_USB, USB_SYSCTL, 0, 31    },
	/*RTD2831_RMAP_INDEX_USB_EPA_CFG      */		{ RTD2831U_USB, USB_EPA_CFG, 0, 31   },
	/* RTD2831_RMAP_INDEX_USB_EPA_MAXPKT*/	{ RTD2831U_USB, USB_EPA_MAXPKT, 0, 31   },
	/*RTD2831_RMAP_INDEX_USB_EPA_FIFO_CFG*/	{ RTD2831U_USB, USB_EPA_FIFO_CFG, 0, 31    },

	/* RTD2831_RMAP_INDEX_SYS_GPD*/			{ RTD2831U_SYS, GPD, 0, 7   },
	/* RTD2831_RMAP_INDEX_SYS_GPOE*/			{ RTD2831U_SYS, GPOE, 0, 7   },
	/* RTD2831_RMAP_INDEX_SYS_GPO*/			{ RTD2831U_SYS, GPO, 0, 7   },
	/* RTD2831_RMAP_INDEX_SYS_SYS_0*/			{ RTD2831U_SYS, SYS_0, 0, 7   },
    
};                                          



static int rtd2830_reg_mask[32]= {
    0x00000001,
    0x00000003,
    0x00000007,
    0x0000000f,
    0x0000001f,
    0x0000003f,
    0x0000007f,
    0x000000ff,
    0x000001ff,
    0x000003ff,
    0x000007ff,
    0x00000fff,
    0x00001fff,
    0x00003fff,
    0x00007fff,
    0x0000ffff,
    0x0001ffff,
    0x0003ffff,
    0x0007ffff,
    0x000fffff,
    0x001fffff,
    0x003fffff,
    0x007fffff,
    0x00ffffff,
    0x01ffffff,
    0x03ffffff,
    0x07ffffff,
    0x0fffffff,
    0x1fffffff,
    0x3fffffff,
    0x7fffffff,
    0xffffffff
};


static void platform_wait(unsigned long nMinDelayTime)
{
	// The unit of Sleep() waiting time is millisecond.

	unsigned long usec;
	do {
		usec = (nMinDelayTime > 8000) ? 8000 : nMinDelayTime;
		msleep(usec);
		nMinDelayTime -= usec;
		} while (nMinDelayTime > 0);

	return;
	
}



static void rtd2830_platform_wait(BASE_INTERFACE_MODULE *pBaseInterface,unsigned long WaitTimeMs)
{
	platform_wait(WaitTimeMs);
	return;
	
}




static int
rtd2830_read_multi_byte(
    struct rtd2830_state*       p_state,
    int                         page_addr,
    int                         byte_addr,
    int                         n_bytes,
    int*                        p_val)
{
	int                         i;
	u8                         val;
	int                         nbit_shift; 

	*p_val= 0;
	
	 if( n_bytes > 4 )
	 {
            	printk("%s: error length = %d\n", __FUNCTION__, n_bytes);
		return 1;
	 }

	for (i= 0; i< n_bytes; i++)
	{
		val= 0;
		
		RTD2831_READ_BYTES(p_state->dib , page_addr , byte_addr+i , &val , 1 );
		
        	if(page_addr<RTD2831U_USB)
		{
			*p_val= (*p_val<< 8)+ val;
		}
		else
		{
			nbit_shift= i<<3 ;
			*p_val= *p_val+ (val<<nbit_shift);
		}
       }
    return 0;
}

static int
rtd2830_write_multi_byte(
    struct rtd2830_state*       p_state,
    int                         page_addr,
    int                         byte_addr,
    int                         n_bytes,
    int                         val)
{
	int                         i;
	int                         nbit_shift;
	u8                          u8_val;				

	for (i= n_bytes- 1; i>= 0; i--)
	{
		if(page_addr<RTD2831U_USB)
			nbit_shift= (n_bytes- i- 1) << 3;
		else//RTD2831U_USB or RTD2831U_SYS
			nbit_shift= i << 3;
		
		u8_val = (val>> nbit_shift) & 0xff;
		if(RTD2831_WRITE_BYTES( p_state->dib , page_addr, byte_addr+ i, &u8_val , 1 ))
			return 1;				
    	}
	
    return 0;
}

static int
rtd2830_read_register(
    struct rtd2830_state*       p_state,
    rtd2830_reg_map_index       reg_map_index,
    int*                        p_val)
{
	int                         page_addr=      rtd2830_reg_map[reg_map_index].page_addr;
	int                         reg_addr=       rtd2830_reg_map[reg_map_index].reg_addr;
	int                         bit_low=        rtd2830_reg_map[reg_map_index].bit_low;
	int                         bit_high=       rtd2830_reg_map[reg_map_index].bit_high;
	int                         n_byte_read=    (bit_high>> 3)+ 1;

	*p_val= 0;
	if (rtd2830_read_multi_byte(p_state, page_addr, reg_addr, n_byte_read, p_val))
	{
		printk("%s: error\n", __FUNCTION__);
		return 1;
	}

	*p_val= ((*p_val>> bit_low) & rtd2830_reg_mask[bit_high- bit_low]);
 
	return 0;
}

static int
rtd2830_write_register(
    struct rtd2830_state*       p_state,
    rtd2830_reg_map_index       reg_map_index,
    int                         val_write)
{
	int                         page_addr=      rtd2830_reg_map[reg_map_index].page_addr;
	int                         reg_addr=       rtd2830_reg_map[reg_map_index].reg_addr;
	int                         bit_low=        rtd2830_reg_map[reg_map_index].bit_low;
	int                         bit_high=       rtd2830_reg_map[reg_map_index].bit_high;
	int                         n_byte_write=   (bit_high>> 3)+ 1;
	int                         val_read= 0;
	int                         new_val_write;

	if (rtd2830_read_multi_byte(p_state, page_addr, reg_addr, n_byte_write, &val_read))
	{
		printk("%s: error in read\n", __FUNCTION__);
		return 1;
	}

	new_val_write= (val_read & (~(rtd2830_reg_mask[bit_high- bit_low]<< bit_low))) | (val_write<< bit_low);

	if (rtd2830_write_multi_byte(p_state, page_addr, reg_addr, n_byte_write, new_val_write))
	{
		printk("%s: error in write\n", __FUNCTION__);
		return 2;
	}
	return 0;
}

/*
static int
rtd2830_write_register_table(
    struct rtd2830_state*       p_state,
    rtd2830_reg_map_index       reg_map_index,
    const u8*                   p_table,
    int                         len_table)
{
	int                         page_addr=      rtd2830_reg_map[reg_map_index].page_addr;
	int                         reg_addr=       rtd2830_reg_map[reg_map_index].reg_addr;
	int                         i;
	printk("+ %s\n", __FUNCTION__);
	for (i= 0; i< len_table; i++)
	{
		if(RTD2831_WRITE_BYTES( p_state->dib , page_addr, reg_addr+ i, &p_table[i] , 1 ))
			return 1;		
	
 	}
	printk("- %s\n", __FUNCTION__);
	return 0;
}

*/


static int
rtl2830_read_register_bytes(
	DVBT_DEMOD_MODULE  *pDemod,
	unsigned char PageNo,
	unsigned char RegStartAddr,
	unsigned char *pReadingBytes,
	unsigned char ByteNum
	)
{
	unsigned int	i;
	void*	tmp_p_state=NULL;
	struct rtd2830_state*       p_state=NULL;	

	pDemod->pBaseInterface->GetUserDefinedDataPointer(pDemod->pBaseInterface, &tmp_p_state); //get user define data p_state
	p_state = (struct rtd2830_state*)tmp_p_state;

	if (ByteNum == 2)  //some registers need to read two address continuously in one read operation
	{
		if(RTD2831_READ_BYTES(p_state->dib , PageNo , RegStartAddr, pReadingBytes , ByteNum ))
			goto  error;	
	}
	else
	{		
		for (i= 0; i< ByteNum; i++)
		{
			if(RTD2831_READ_BYTES(p_state->dib , PageNo , RegStartAddr+i , &pReadingBytes[i] , 1 ))
				goto  error;			
		}
	}
	return FUNCTION_SUCCESS;

error:
	return FUNCTION_ERROR;
	
}


static int
rtl2830_write_register_bytes(
	DVBT_DEMOD_MODULE  *pDemod,
	unsigned char PageNo,
	unsigned char RegStartAddr,
	const unsigned char *pWritingBytes,
	unsigned char ByteNum
	)
{

	unsigned int	i;
	void*				tmp_p_state=NULL;				
	struct rtd2830_state*	p_state=NULL;
	u8			tmp_buf;
	
	pDemod->pBaseInterface->GetUserDefinedDataPointer(pDemod->pBaseInterface, &tmp_p_state); //get user define data p_state
	p_state = (struct rtd2830_state*)tmp_p_state; 
			
	for (i= 0; i< ByteNum; i++)
	{
		tmp_buf = pWritingBytes[i];
		if(RTD2831_WRITE_BYTES( p_state->dib , PageNo , RegStartAddr+i , &tmp_buf , 1 ))
			goto  error;		
	}

	return FUNCTION_SUCCESS;
error:
	return FUNCTION_ERROR;

}



int
rtd2830_soft_reset(
    struct rtd2830_state*       p_state)
{
	printk("+%s\n", __FUNCTION__);

	p_state->pDemod->SoftwareReset(p_state->pDemod);

	printk("-%s\n", __FUNCTION__);
	return 0;
}


int
rtd2830_general_init(
    struct rtd2830_state*       p_state)
{
	int Status;

	printk("###################%s:\n", __FUNCTION__);

	 if (p_state->tuner_type == RTDUSB_TUNER_MXL5005)
	{
		//MXL Tuner Setting
		Status = p_state->pDemod->Initialize(p_state->pDemod, RTL2830_APPLICATION_DONGLE, IF_FREQ_4570000HZ, SPECTRUM_NORMAL, 0x3F, 0x4);
		Status = p_state->pDemod->SetRegBits(p_state->pDemod, DVBT_AGC_TARG_VAL , 0x3e);
		printk("%s : demod know tuner type MXL5005\n", __FUNCTION__);
	}
	else if (p_state->tuner_type == RTDUSB_TUNER_MT2060)
	{
		//MT2060 Setting
		Status = p_state->pDemod->Initialize(p_state->pDemod, RTL2830_APPLICATION_DONGLE, IF_FREQ_36125000HZ, SPECTRUM_INVERSE, 0x20, 0x4);
		printk("%s : demod know tuner type MT2060\n", __FUNCTION__);
	}
	else
	{
		printk("%s : tuner type error!!\n", __FUNCTION__);
	}

	printk("%s\n", __FUNCTION__);

	return 0;
}

static int rtd2830_read_status(struct dvb_frontend* fe, fe_status_t* status)
{
	u8						rtd2830_fsm_stage= -1;
	struct rtd2830_state*      	state= fe->demodulator_priv;

	if (down_interruptible(&state->mutex_rtd2831))
		return -ERESTARTSYS;	

	*status = 0;

	state->pDemod->GetFsmStage(state->pDemod , &rtd2830_fsm_stage);	

	printk("  %s ******FSM = %d******\n", __FUNCTION__, rtd2830_fsm_stage);

	if (0x0b== rtd2830_fsm_stage)
		*status|= (FE_HAS_CARRIER| FE_HAS_VITERBI| FE_HAS_LOCK| FE_HAS_SYNC| FE_HAS_SIGNAL);

	up( &state->mutex_rtd2831 );
	
	return 0;
}

static int
rtd2830_set_bandwidth(
    struct rtd2830_state*       p_state,
    enum fe_bandwidth           bandwidth)
{

	switch (bandwidth) 
	{
	case BANDWIDTH_6_MHZ:
		p_state->pDemod->SetBandwidthMode(p_state->pDemod, DVBT_BANDWIDTH_6MHZ);	
		break;
		
	case BANDWIDTH_7_MHZ:
		p_state->pDemod->SetBandwidthMode(p_state->pDemod, DVBT_BANDWIDTH_7MHZ);	
		break;
		
	case BANDWIDTH_8_MHZ:
	default:
		p_state->pDemod->SetBandwidthMode(p_state->pDemod, DVBT_BANDWIDTH_8MHZ);			   
		break;
	}	
	return 0;
}



void
rtd2830_update_functions(struct work_struct *work)
{
	struct rtd2830_state* p_state = container_of( work , struct rtd2830_state , update_procedure_work.work); 

	if (down_interruptible(&p_state->mutex_rtd2831))
		return;
	
	p_state->pDemod->UpdateFunction2(p_state->pDemod);
	p_state->pDemod->UpdateFunction3(p_state->pDemod);
	p_state->pDemod->UpdateFunction4(p_state->pDemod);
	p_state->pDemod->UpdateFunction5(p_state->pDemod);

	schedule_delayed_work(&p_state->update_procedure_work,UPDATE_PROCEDURE_PERIOD);

	up( &p_state->mutex_rtd2831 );

	return;
}



static int 
rtd2830_init(
    struct dvb_frontend*        fe)
{
	struct rtd2830_state*	p_state = fe->demodulator_priv;
	unsigned char			pid_data[] = { 208, 1 , 0 , 0 , 0 , 31 , 255 };

	
	int data;

	if (down_interruptible ( &p_state->mutex_rtd2831 ) )
		return -ERESTARTSYS;
	

	printk("+ ########################## %s: %d: init once\n", __FUNCTION__, __LINE__);

	p_state->b_initial = FALSE;

	//2 get info :HIGH_SPEED or FULL_SPEED

	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_STAT, &data);
	printk("#####################################################TD2831_RMAP_INDEX_USB_STAT=0x%x\n",data);
	if(data&0x01)  
	{
		printk("HIGH SPEED\n");
	}
	else 
    	{
		printk("FULL SPEED\n");
	}

#if 1 
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_SYS_GPD, 0x0a);
	
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_SYS_GPOE, 0x15);

	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_SYS_GPO, &data);
	printk("#####################################################RTD2831_RMAP_INDEX_SYS_GPO=0x%x\n",data);      
	
	data |= 0x01;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_SYS_GPO,data);

	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_SYS_GPO, &data);
	printk("#####################################################RTD2831_RMAP_INDEX_SYS_GPO=0x%x\n",data);            

	data |= 0x04;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_SYS_GPO,data);
	
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_SYS_GPO, &data);
	printk("#####################################################RTD2831_RMAP_INDEX_SYS_GPO=0x%x\n",data);            

	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_SYS_GPO,data);
#endif

	//2 1.open demod pll enbale
	//2 2.open demod adc enbale
	//2 3.change the add of internal demod module

	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_SYS_SYS_0, &data);

	data |= 0xe0;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_SYS_SYS_0,data);

	data =0xcc;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_SYS_SYS_0, &data);

	if((data & 0xe0) != 0xe0)
	{
		printk("Write DEMOD_CTL Register error!!\n");
	 	goto error;
	}

	//2 1.FULL packer mode(for bulk)
	//2 2.verification mode(will not used in future)
	//2 3.DMA enable.
	
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_SYSCTL, &data);

	data &=0xffffff00;
	data |= 0x09;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_USB_SYSCTL, data);

	data=0;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_SYSCTL, &data);
      
	if((data&0xff)!=0x09)  
	{
		printk("Open bulk FULL packet mode error!!\n");
	 	goto error;
	}

	//2check epa config,
	//2[9-8]:00, 1 transaction per microframe
	//2[7]:1, epa enable
	//2[6-5]:10, bulk mode
	//2[4]:1, device to host
	//2[3:0]:0001, endpoint number

	data = 0;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_CFG, &data);                 
	if((data&0x0300)!=0x0000 || (data&0xff)!=0xd1)
	{
		printk("Open bulk EPA config error! data=0x%x \n" , data);
	 	goto error;	
	}

	//2 EPA maxsize packet 
	//2 512:highspeedbulk, 64:fullspeedbulk. 
	//2 940:highspeediso,  940:fullspeediso.

	data = 0x00000200;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_MAXPKT, data);

	data=0;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_MAXPKT, &data);
                      
	if((data&0xffff)!=0x0200)
	{
		printk("Open bulk EPA max packet size error!\n");
	 	goto error;
	}

	//2 change fifo length of EPA 

	data = 0x00000014;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_FIFO_CFG, data);

	data = 0xcccccccc;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_FIFO_CFG, &data);

	if( (data & 0xff) != 0x14)
	{
		printk("Write error RTD2831_RMAP_INDEX_USB_EPA_FIFO_CFG =0x%x\n",data);
	 	goto error;
	}


	//2reset epa fifo

	data = 0x0200;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_CTL,data);

	data = 0xffff;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_CTL,&data);

	if( (data & 0xffff) != 0x0200)
	{
		printk("Write error RTD2831_RMAP_INDEX_USB_EPA_CTL = 0x%x\n",data);
	 	goto error;	
	}

	data=0x0000;
	rtd2830_write_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_CTL,data);

	data = 0xffff;
	rtd2830_read_register(p_state, RTD2831_RMAP_INDEX_USB_EPA_CTL,&data);

	if( ( data  & 0xffff) != 0x0000)
	{
		printk("Write error RTD2831_RMAP_INDEX_USB_EPA_CTL = 0x%x\n",data);
	 	goto error;	
	}
    
	//2 set pid filter in reject mode 	
	rtl2830_write_register_bytes( p_state->pDemod, 0 , 0x61 , pid_data ,  sizeof (pid_data) /sizeof (unsigned char) );


	//2 tuner initialize
	rtdusb_general_tuner_init(fe);
	


	if (rtd2830_general_init(p_state))
	{	 
		printk("rtd2830_general_init fail\n");
	 	goto error;
	}

	p_state->pDemod->SoftwareReset(p_state->pDemod);

	// Delay 1000 ms	
	platform_wait(1000);
	

	schedule_delayed_work(&p_state->update_procedure_work,UPDATE_PROCEDURE_PERIOD);


	p_state->b_initial = TRUE;	
	up( &p_state->mutex_rtd2831 );    
	printk("- %s\n", __FUNCTION__);
	return 0;

error:
	up( &p_state->mutex_rtd2831 );
	return 1;
	
}






static void
rtd2830_release(
    struct dvb_frontend*            fe)
{				
	struct rtd2830_state* state = fe->demodulator_priv;

	if (down_interruptible ( &state->mutex_rtd2831 ))
		return;

	printk("+ %s\n", __FUNCTION__);

	if( state->b_initial )
	{
		state->b_initial = FALSE;
		cancel_rearming_delayed_work( &(state->update_procedure_work) );
		flush_scheduled_work();
	}

	//2  release tuner 
	rtdusb_general_tuner_release(fe);

	kfree(state);	

	printk("- %s\n", __FUNCTION__);
	up( &state->mutex_rtd2831 );
}



static void
rtd2830_set_stage_pass(
	struct rtd2830_state*	p_state,
	unsigned int			offset_bit,
	unsigned int			onoff)
{				
	unsigned long Value;
	
	if( offset_bit < 0 || offset_bit > 11)
	{
		printk("%s : offset_bit = 0x%x error!\n", __FUNCTION__ , offset_bit);
		return ;
	}

	if( (onoff != 0) && (onoff != 1) )
	{
		printk("%s : onoff = 0x%x error!\n", __FUNCTION__ , onoff);
		return ;
	}

	p_state->pDemod->GetRegBits(p_state->pDemod, DVBT_SM_BYPASS, &Value);

	if(onoff == 1)		Value = Value|(0x01<<offset_bit);
	else				Value =  Value&~(0x01<<offset_bit);	

	p_state->pDemod->SetRegBits(p_state->pDemod, DVBT_SM_BYPASS, Value); 
//	printk("%s :  after SM_BYPASS = 0x%x \n", __FUNCTION__ , Value);
	return ;

}



static void
rtd2830_scan_channel_procedure(
	struct dvb_frontend*            fe,
	struct dvb_frontend_parameters* param)
{
	struct rtd2830_state*	p_state= fe->demodulator_priv;
	unsigned long			diff_ms;
	unsigned char			FsmStage;		
	unsigned char			FftMode , TPS_lock_status , signal_lock_status;	

	unsigned long			frequency = param->frequency;
	const long			frequency_offset =167000;	
	unsigned long			frequency_offset_table[] = {	frequency + frequency_offset ,
													frequency - frequency_offset };
	unsigned int			frequency_offset_times = sizeof(frequency_offset_table) /sizeof(unsigned long);

	
	rtd2830_set_stage_pass( p_state , 1 , 0 );
	rtd2830_set_stage_pass( p_state , 7 , 0 );
	rtd2830_soft_reset( p_state );
	platform_wait(40);
	rtd2830_set_stage_pass( p_state , 1 , 1 );
	platform_wait(40);
	p_state->pDemod->GetFsmStage(p_state->pDemod , &FsmStage);	

	diff_ms = 0;						
	while(!(FsmStage >= 5 || diff_ms > 1000))
	{
		platform_wait(40);
		diff_ms += 40;
		p_state->pDemod->GetFsmStage(p_state->pDemod , &FsmStage);	
	}
			
	if(FsmStage < 5)
	{
		printk("%s : FsmStage < 5 \n", __FUNCTION__ );
		FftMode = DVBT_FFT_MODE_8K;
		goto frequency_excursion;
	}
			
	p_state->pDemod->GetFftMode(p_state->pDemod , &FftMode); 
	rtd2830_set_stage_pass( p_state , 7 , 1 );

scan_channel_procedure:
	
	p_state->pDemod->TestTpsLock(p_state->pDemod , &TPS_lock_status);
		
	diff_ms = 0;	
	while(! (TPS_lock_status == LOCKED || diff_ms > 1000) )
	{
		platform_wait(40);
		diff_ms += 40;
		p_state->pDemod->TestTpsLock(p_state->pDemod , &TPS_lock_status);			
	}
	

	if(TPS_lock_status == NOT_LOCKED )
	{
		printk("%s : *****TPS_NON_LOCK***** \n", __FUNCTION__ );
		goto frequency_excursion;
	}

	rtd2830_set_stage_pass( p_state , 11 , 0 );
	p_state->pDemod->TestSignalLock( p_state->pDemod ,  &signal_lock_status);
		
	diff_ms = 0;		
	while( !(signal_lock_status == LOCKED || diff_ms > 1600) )
	{
		platform_wait(40);
		diff_ms += 40;
		p_state->pDemod->TestSignalLock( p_state->pDemod ,  &signal_lock_status);	
	}

	rtd2830_set_stage_pass( p_state , 11 , 1 );	

	if(signal_lock_status == LOCKED)
	{
		printk("%s : *****SIGNAL_LOCK***** \n", __FUNCTION__ );
		return ;    //2SIGNAL_LOCK	
	}
	else
	{
		printk("%s : *****SIGNAL_NON_LOCK***** \n", __FUNCTION__ );
		return ;    //2SIGNAL_NON_LOCK	
	}	

frequency_excursion:

	if( (FftMode == DVBT_FFT_MODE_2K )||(p_state->frequency_excursion_times >= frequency_offset_times) ) 
	{
		printk("%s : ***** Signal NOT Present***** \n", __FUNCTION__ );
		return ;	   //2 Signal NOT Present
	}
	else
	{		
		param->frequency = frequency_offset_table[p_state->frequency_excursion_times];	
		rtdusb_general_tuner_set(fe,param);
		p_state->frequency_excursion_times++;
		rtd2830_soft_reset( p_state );
		goto scan_channel_procedure;
	}	

}





static int
rtd2830_set_parameters(
    struct dvb_frontend*            fe,
    struct dvb_frontend_parameters* param)
{
	struct rtd2830_state*			p_state= fe->demodulator_priv;
	struct dvb_ofdm_parameters*	p_ofdm_param= &param->u.ofdm;
	unsigned long					frequency = param->frequency;	

#ifdef RTL2831Debug
	int							page, offset;
	unsigned char					ReadingByte;	
#endif	

	if (down_interruptible ( &p_state->mutex_rtd2831 ))
		return -ERESTARTSYS;
	  
	printk("+ %s\n", __FUNCTION__);	

	if (0== memcmp(&p_state->fep, param, sizeof(struct dvb_frontend_parameters)))
	{
		up( &p_state->mutex_rtd2831 );
		return 1;
	}
	memcpy(&p_state->fep, param, sizeof(struct dvb_frontend_parameters));
	rtd2830_set_bandwidth(p_state, p_ofdm_param->bandwidth);
	
	p_state->frequency_excursion_times = 0;
	rtdusb_general_tuner_set(fe, param);
	
	//2 scan channel
	rtd2830_scan_channel_procedure( fe ,param );
	p_state->pDemod->ResetFunction4(p_state->pDemod);
	p_state->pDemod->ResetFunction5(p_state->pDemod);


#ifdef RTL2831Debug
	printk("+Demod reg dump\n");

	for( page = 0 ; page < 3; page++)
	{
		for( offset = 0; offset < 256; offset++)
		{
			p_state->pDemod->GetRegBytes(p_state->pDemod, page , offset , &ReadingByte, LEN_1_BYTE);
			printk("%d, %d, %d\n", page , offset , ReadingByte);
		}
	}
	printk("-Demod reg dump\n");
#endif

	param->frequency = frequency;
	
	printk("- %s\n", __FUNCTION__);
	
	up( &p_state->mutex_rtd2831 );	

	return 0;
}



static int rtd2830_get_parameters(
	struct dvb_frontend* 			fe,
	struct dvb_frontend_parameters*fep)
{
	struct rtd2830_state* state = fe->demodulator_priv;
	memcpy(fep,&state->fep,sizeof(struct dvb_frontend_parameters));
	return 0;
}

static int rtd2830_sleep(struct dvb_frontend* fe)
{
    printk("%s: Do Nothing\n",__FUNCTION__);
    return 0;
}

static int rtd2830_read_ber(struct dvb_frontend* fe, u32* ber)
{
	struct rtd2830_state* p_state = fe->demodulator_priv;
	unsigned  long ber_num, ber_dem;
	
	if (down_interruptible(&p_state->mutex_rtd2831))
		return -ERESTARTSYS;
	
	p_state->pDemod->GetBer( p_state->pDemod , &ber_num , &ber_dem );

	*ber = ber_num ;
	
	printk( "%s: ber =%d \n",__FUNCTION__, *ber);

	up( &p_state->mutex_rtd2831 );	
	return 0;
}

static int rtd2830_read_signal_strength(struct dvb_frontend* fe, u16* strength)
{
	struct rtd2830_state* p_state = fe->demodulator_priv;
	unsigned char  _strength;
	
	if (down_interruptible(&p_state->mutex_rtd2831))
		return -ERESTARTSYS;	
	
	p_state->pDemod->GetSignalStrength( p_state->pDemod,  &_strength );

	*strength =  _strength;
	
	printk( "%s: signal strength =%d \n",__FUNCTION__, *strength);

	up( &p_state->mutex_rtd2831 );
	
	return 0;
}

static int rtd2830_read_snr(struct dvb_frontend* fe, u16* snr)
{
	struct rtd2830_state* p_state = fe->demodulator_priv;
	long snr_num, snr_dem , _snr;

	if (down_interruptible(&p_state->mutex_rtd2831))
		return -ERESTARTSYS;
	
	p_state->pDemod->GetSnr( p_state->pDemod , &snr_num , &snr_dem );

	_snr = snr_num / snr_dem;

	if( _snr < 0 ) _snr = 0;

	*snr = _snr;

	printk( "%s: ***snr =%d*** \n",__FUNCTION__, (int) *snr); 

	up( &p_state->mutex_rtd2831 );
	
	return 0;
}

static int rtd2830_read_ucblocks(struct dvb_frontend* fe, u32* ucblocks)
{
    printk("%s: Do Nothing\n",__FUNCTION__); 
    return 0;
}

static int rtd2830_get_tune_settings(struct dvb_frontend* fe, struct dvb_frontend_tune_settings* fe_tune_settings)
{
    printk("%s: Do Nothing\n",__FUNCTION__);
    return 0;
}




static struct dvb_frontend_ops rtd2830_ops;

struct dvb_frontend* rtd2831u_fe_attach(struct dvb_usb_device *d)
{

	struct rtd2830_state*       state= NULL;

	printk("+%s\n", __FUNCTION__);
	 
    /* allocate memory for the internal state */
	state = kzalloc(sizeof(struct rtd2830_state), GFP_KERNEL);
	if (state == NULL) goto error;
	memset(state,0,sizeof(*state));

	state->dib = d;

 	// Build base interface module.
	BuildBaseInterface(&state->pBaseInterface,
					    &state->BaseInterfaceModuleMemory,
					    0,			//not needed
					    0,			//not needed
					    NULL,		//not needed
					    NULL,		//not needed
					    rtd2830_platform_wait);

	// Build RTL2830 DVB-T demod module.
	BuildRtl2830Module(
		&state->pDemod,
		&state->DvbtDemodModuleMemory,
		&state->BaseInterfaceModuleMemory,
		&state->I2cBridgeModuleMemory,
		0x20,								// I2C device address
		UPDATE_PROCEDURE_PERIOD			// Function 4 update period (unit: ms)
		);

	state->pDemod->SetRegBytes = rtl2830_write_register_bytes;
	state->pDemod->GetRegBytes= rtl2830_read_register_bytes;
	

	/* setup the state */
	memcpy(&state->frontend.ops, &rtd2830_ops, sizeof(struct dvb_frontend_ops));
	memset(&state->fep, 0, sizeof(struct dvb_frontend_parameters));

	state->b_initial = FALSE;
    
 	/* create dvb_frontend */
	state->frontend.demodulator_priv = state;
	state->frequency_excursion_times = 0;

	state->tuner_type = RTDUSB_NO_TUNER;

	state->pDemod->pBaseInterface->SetUserDefinedDataPointer(state->pDemod->pBaseInterface, (void *)state); //set user define data state to demod


	INIT_DELAYED_WORK( &(state->update_procedure_work), rtd2830_update_functions);

	sema_init( &state->mutex_rtd2831 , 1 );

	printk("-%s\n", __FUNCTION__);
	return &state->frontend;

error:	
	kfree(state);
	return NULL;
}




static struct dvb_frontend_ops rtd2830_ops = {
    .info = {
        .name               = "Realtek RTD2830 DVB-T",
        .type               = FE_OFDM,
        .frequency_min      = 174000000,
        .frequency_max      = 862000000,
        .frequency_stepsize = 166667,
        .caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 |
            FE_CAN_FEC_3_4 | FE_CAN_FEC_5_6 | FE_CAN_FEC_7_8 |
            FE_CAN_FEC_AUTO |
            FE_CAN_QPSK | FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_QAM_AUTO |
            FE_CAN_TRANSMISSION_MODE_AUTO | FE_CAN_GUARD_INTERVAL_AUTO |
            FE_CAN_HIERARCHY_AUTO | FE_CAN_RECOVER |
            FE_CAN_MUTE_TS
    },

    .init =					rtd2830_init,
    .release =				rtd2830_release,

    .sleep =				rtd2830_sleep,

    .set_frontend =			rtd2830_set_parameters,
    .get_frontend =			rtd2830_get_parameters,
    .get_tune_settings =		rtd2830_get_tune_settings,

    .read_status =			rtd2830_read_status,
    .read_ber =			rtd2830_read_ber,
    .read_signal_strength =	rtd2830_read_signal_strength,
    .read_snr =			rtd2830_read_snr,
    .read_ucblocks =		rtd2830_read_ucblocks,
};


