
#include "rtd2831u.h"
#include "tuner_demod_io.h"

/* debug */
int dvb_usb_rtd2831u_debug;
module_param_named(debug,dvb_usb_rtd2831u_debug, int, 0644);
MODULE_PARM_DESC(debug, "set debugging level (1=info,xfer=2 (or-able))." DVB_USB_DEBUG_STATUS);


#define	RTL2831U_NEC_PROTOCOL
//#define	RTL2831U_RC5_PROTOCOL


#define IRRC_RP 			0x0034			//IR read port 
#define IRRC_SR 			0x0038			//IR status register 


#ifdef RTL2831U_NEC_PROTOCOL
static struct dvb_usb_rc_key rtd2831u_nec_keys [] =
{
	{  0x86, 0x79, KEY_A },
	{  0x93, 0x6c, KEY_B },
	{  0x8f, 0x70, KEY_UP },
	{  0x9d, 0x62, KEY_DOWN},
};
#else 
#ifdef RTL2831U_RC5_PROTOCOL
/* philip rc5 keys */
static struct dvb_usb_rc_key rtd2831u_rc5_keys [] = {
	{ 0x00, 0x23, KEY_1 },
	{ 0x00, 0xa3, KEY_2 },
	{ 0x00, 0x63, KEY_3 },
	{ 0x00, 0x13, KEY_4 },
	{ 0x00, 0x93, KEY_A },
};
#endif
#endif




static int rtd2831u_rc_init(struct dvb_usb_device *d)
{

	unsigned short IR_REG_ADDR[14] = {0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x28,0x29,0x2c,0x2e,0x30,0x31,0x33}; //for ir initial
#ifdef RTL2831U_NEC_PROTOCOL
	u8 NEC_REG_VAL[14] = {0x43, 0x16, 0x16, 0x5a, 0x2d,0x16, 0x01, 0xb0, 0x04, 0x88,0x13, 0xdf, 0x05, 0x80};
#else 
#ifdef RTL2831U_RC5_PROTOCOL
	u8 RC5_REG_VAL[14] = {0x24, 0x24, 0x24, 0x00, 0x00,0x00, 0x01, 0xb0, 0x04, 0xff,0x03, 0x0c, 0x05, 0x80};
#endif
#endif
	u8 reg_val[14];
	u8 read_data = 0xcc;
	u8 i;	
	struct rtl2831u_rc_state* p_rc_state = d->priv;


	memset(reg_val,0,sizeof(u8)*ARRAY_SIZE(reg_val));		
#ifdef RTL2831U_NEC_PROTOCOL
	memcpy(reg_val,NEC_REG_VAL,sizeof(u8)*ARRAY_SIZE(reg_val));
#else 
#ifdef RTL2831U_RC5_PROTOCOL
	memcpy(reg_val,RC5_REG_VAL,sizeof(u8)*ARRAY_SIZE(reg_val));
#endif
#endif

	RTD2831_WRITE_BYTES( d, RTD2831U_SYS , IR_REG_ADDR[13], &reg_val[13], 1);

	for (i=0; i<sizeof(IR_REG_ADDR)/sizeof(IR_REG_ADDR[0])-1 ; i++)
	{
		read_data = 0xcc;
		RTD2831_WRITE_BYTES( d , RTD2831U_SYS , IR_REG_ADDR[i], &reg_val[i], 1);			
	}

	p_rc_state->b_rc_initial = 1; //IR initial
	
	return 0;
}


static int rtd2831u_rc_key_to_event(struct dvb_usb_device *d,
		u8 keybuf[4], u32 *event, int *state)
{
	int i;
	struct dvb_usb_rc_key *keymap = d->props.rc_key_map;
	*event = 0;
	*state = REMOTE_NO_KEY_PRESSED;

#ifdef RTL2831U_RC5_PROTOCOL
	keybuf[0] = (keybuf[0] & 0x00);
	keybuf[1] = (keybuf[1] & 0x00);
	keybuf[2] = (keybuf[2] & 0x00);
	keybuf[3] = (keybuf[3] & 0xff);
#endif

//	printk("raw key code 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",keybuf[0], keybuf[1], keybuf[2], keybuf[3]);

	/* See if we can match the raw key code. */
	for (i = 0; i < d->props.rc_key_map_size; i++)
		if (keymap[i].custom == keybuf[2] && keymap[i].data == keybuf[3]) 
		{
			*event = keymap[i].event;
			*state = REMOTE_KEY_PRESSED;
			return 0;
		}
	
	return 0;

}

static int rtd2831u_rc_query(struct dvb_usb_device *d, u32 *event, int *state)
{
	u8 key[4];
	u8 b = 0xcc;
	struct rtl2831u_rc_state* p_rc_state = d->priv;
		
	*event = 0;
	*state = REMOTE_NO_KEY_PRESSED;	

	if(!p_rc_state->b_rc_initial)	//not IR initial, so initial IR 070713 
	{
		rtd2831u_rc_init(d);
	}

	RTD2831_READ_BYTES( d , RTD2831U_SYS , IRRC_SR , &b , 1);
	if(b&0x01)
	{
		RTD2831_READ_BYTES( d , RTD2831U_SYS , IRRC_RP , key , 4);
		b = 0x01;	
		RTD2831_WRITE_BYTES(d , RTD2831U_SYS , IRRC_SR , &b , 1);	

		if( ++p_rc_state->repeat_key_count < p_rc_state->rc_key_repeat_count )
			return 0; //set two events to 1(repeat.....)

		rtd2831u_rc_key_to_event( d , key , event , state );
		p_rc_state->repeat_key_count = 0 ; //reset counter to 0		
	}
	
	return 0;
}

static int rtd2831u_frontend_attach(struct dvb_usb_adapter *adap)
{
	struct rtl2831u_rc_state* p_rc_state = adap->dev->priv;

	p_rc_state->b_rc_initial = 0;		//not IR initial 070713
	p_rc_state->repeat_key_count = 0;	//counter received IR code times
	p_rc_state->rc_key_repeat_count = 2;	//set 2 events to 1 event	
	adap->fe = rtd2831u_fe_attach(adap->dev); 
	return 0;
}

static struct dvb_usb_device_properties rtd2831u_properties;
static struct dvb_usb_device_properties haihua_properties;

static int rtd2831u_usb_probe(struct usb_interface *intf,
		const struct usb_device_id *id)
{
	if (dvb_usb_device_init(intf,&rtd2831u_properties,THIS_MODULE,NULL) == 0 ||
	    dvb_usb_device_init(intf,&haihua_properties,THIS_MODULE,NULL) == 0 )
		return 0;

	return -ENODEV;
}

static struct usb_device_id rtd2831u_usb_table [] = {
	{ USB_DEVICE(USB_VID_REALTEK,      USB_PID_RTD2831U_WARM) },
	{ USB_DEVICE(USB_VID_REALTEK_2,  USB_PID_RTD2831U_WARM_2) },
	{ USB_DEVICE(USB_VID_HAIHUA,       USB_PID_HAIHUAUSB_WARM) },
	{ USB_DEVICE(USB_VID_HAIHUA,       USB_PID_HAIHUAMIN_WARM) },
	{ USB_DEVICE(USB_VID_HAIHUA,       USB_PID_HAIHUAA_WARM) },
	{ USB_DEVICE(USB_VID_HAIHUA,       USB_PID_HAIHUAL_WARM) },
	{ USB_DEVICE(USB_VID_HAIHUA,       USB_PID_HAIHUAZ_WARM) },
	{ USB_DEVICE(USB_VID_BILLIONTON, USB_PID_BILLIONTON_WARM) },
	{ USB_DEVICE(USB_VID_COMPRO,      USB_PID_COMPRO_WARM) },
	{ USB_DEVICE(USB_VID_VESTEL,        USB_PID_VESTEL_WARM) },
	{ USB_DEVICE(USB_VID_FREECOM,     USB_PID_FREECOM_WARM) },	
	{ 0 },
};


MODULE_DEVICE_TABLE(usb, rtd2831u_usb_table);

static struct dvb_usb_device_properties rtd2831u_properties = {
	.size_of_priv = sizeof(struct rtl2831u_rc_state),
	.num_adapters = 1,
	.adapter = 
	{
		{
			.frontend_attach = rtd2831u_frontend_attach,
			/* parameter for the MPEG2-data transfer */
			.stream = 
			{
				.type = USB_BULK,
				.count = RTD2831_URB_NUMBER,
				.endpoint = 0x01,		//data pipe
				.u = 
				{
					.bulk = 
					{
						.buffersize = RTD2831_URB_SIZE,
					}
				}
			},
		}
	},

	.rc_interval     = 300,
#ifdef RTL2831U_NEC_PROTOCOL	
	.rc_key_map      = rtd2831u_nec_keys,
	.rc_key_map_size = ARRAY_SIZE(rtd2831u_nec_keys),
#else
#ifdef RTL2831U_RC5_PROTOCOL
	.rc_key_map      = rtd2831u_rc5_keys,
	.rc_key_map_size = ARRAY_SIZE(rtd2831u_rc5_keys),
#endif
#endif
	.rc_query        = rtd2831u_rc_query,
	
	.num_device_descs = 6,
	.devices = {
		{ .name = "RTL2831U DVB-T USB2.0 DEVICE",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[0], NULL },
		},
		{ .name = "RTL2831U DVB-T USB2.0 DEVICE",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[1], NULL },
		},
		{ .name = "DVB-T TV-Tuner Card-R",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[7], NULL },
		},
		{ .name = "VideoMate TV U100",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[8], NULL },
		},		
		{ .name = "Vestel DVB-T TV Card",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[9], NULL },
		},
		{ .name = "Freecom USB 2.0 DVB-T Device",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[10], NULL },
		},				
		{ NULL },
	}
};




static struct dvb_usb_device_properties haihua_properties = {
	.size_of_priv = sizeof(struct rtl2831u_rc_state),
	.num_adapters = 1,
	.adapter = 
	{
		{
			.frontend_attach = rtd2831u_frontend_attach,
			/* parameter for the MPEG2-data transfer */
			.stream = 
			{
				.type = USB_BULK,
				.count = RTD2831_URB_NUMBER,
				.endpoint = 0x01,		//data pipe
				.u = 
				{
					.bulk = 
					{
						.buffersize = RTD2831_URB_SIZE,
					}
				}
			},
		}
	},

	.rc_interval     = 300,
#ifdef RTL2831U_NEC_PROTOCOL	
	.rc_key_map      = rtd2831u_nec_keys,
	.rc_key_map_size = ARRAY_SIZE(rtd2831u_nec_keys),
#else
#ifdef RTL2831U_RC5_PROTOCOL
	.rc_key_map      = rtd2831u_rc5_keys,
	.rc_key_map_size = ARRAY_SIZE(rtd2831u_rc5_keys),
#endif
#endif
	.rc_query        = rtd2831u_rc_query,
	
	.num_device_descs = 5,
	.devices = {
		{ .name = "DTV-DVB UDTT 7047-USB 2.0 DVB-T Driver",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[2],NULL },
		},
		{ .name = "DTV-DVB UDTT 7047M-USB 2.0 DVB-T Driver",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[3], NULL },
		},
		{ .name = "DTV-DVB UDTT 7047A-USB 2.0 DVB-T Driver",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[4], NULL },
		},
		{ .name = "DTV-DVB UDTT 704L-USB 2.0 DVB-T Driver",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[5], NULL },
		},
		{ .name = "DTV-DVB UDTT 7047Z-USB 2.0 DVB-T Driver",
		  .cold_ids = { NULL, NULL },
		  .warm_ids = { &rtd2831u_usb_table[6], NULL },
		},
		{ NULL },
	}
};






/* usb specific object needed to register this driver with the usb subsystem */
static struct usb_driver rtd2831u_usb_driver = {
	.name		= "dvb_usb_rtd2831u",
	.probe		= rtd2831u_usb_probe,
	.disconnect = dvb_usb_device_exit,
	.id_table	= rtd2831u_usb_table,
};

/* module stuff */
static int __init rtd2831u_usb_module_init(void)
{
	int result;
	if ((result = usb_register(&rtd2831u_usb_driver))) {
		err("usb_register failed. (%d)",result);
		return result;
	}

	return 0;
}

static void __exit rtd2831u_usb_module_exit(void)
{
	/* deregister this driver from the USB subsystem */
	usb_deregister(&rtd2831u_usb_driver);
}

module_init(rtd2831u_usb_module_init);
module_exit(rtd2831u_usb_module_exit);

MODULE_AUTHOR("ChiaLing");
MODULE_DESCRIPTION("Driver for the RTL2831U DVB-T USB2.0 devices");
MODULE_VERSION("1.0");
MODULE_LICENSE("GPL");
