

#include "tuner.h"
#include "tuner_mxl5005s.h"
#include "MT2060Struct.h"
#include "MT2060Tuner.h"
#include "dvb-usb.h"
#include "rtd2830_priv.h"
#include "tuner_demod_io.h"



//2//////////////////////MT2060 API//////////////////


static int
microtune_mt2060_init(
    struct dvb_frontend*	fe)
{	
	mt2060_tuner_init( fe );
	return 0;
}



static int
microtune_mt2060_set(
    struct dvb_frontend*			fe,
    struct dvb_frontend_parameters*	fep)
{
	set_mt2060_tuner( fe , fep);
	return 0;
}







//2//////////////////////MXL5005 API///////////////////////////


static int
maxlinear_mxl5005_init(
    struct dvb_frontend*	fe)
{
	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	struct dvb_usb_device*	dib = p_state->dib;
	TUNER_MODULE*		pTuner_Mxl5005 = (TUNER_MODULE*)p_state->pTuner;
	MXL5005S_EXTRA_MODULE*	p_mxl5005s_extra = (MXL5005S_EXTRA_MODULE*) pTuner_Mxl5005->pExtra;
			
	pTuner_Mxl5005->Initialize( dib, pTuner_Mxl5005 );
	p_mxl5005s_extra->SetSpectrumMode( dib , pTuner_Mxl5005 , SPECTRUM_NORMAL);
	return 0;
}



static int
maxlinear_mxl5005_set(
    struct dvb_frontend*			fe,
    struct dvb_frontend_parameters*	fep)
{
	struct rtd2830_state*		p_state = (struct rtd2830_state*)fe->demodulator_priv;
	struct dvb_usb_device*		dib = p_state->dib;
	TUNER_MODULE*			pTuner_Mxl5005 = (TUNER_MODULE* )p_state->pTuner;
	MXL5005S_EXTRA_MODULE*		p_mxl5005s_extra = (MXL5005S_EXTRA_MODULE*) pTuner_Mxl5005->pExtra;
	struct dvb_ofdm_parameters*	p_ofdm_param = &fep->u.ofdm;
	unsigned long					actualfreqHz;

	switch (p_ofdm_param->bandwidth) 
	{
	case BANDWIDTH_6_MHZ:		   
		p_mxl5005s_extra->SetBandwidthHz( dib, pTuner_Mxl5005 , MXL5005S_BANDWIDTH_6MHZ);
		break;
		
	case BANDWIDTH_7_MHZ:
		p_mxl5005s_extra->SetBandwidthHz( dib, pTuner_Mxl5005 , MXL5005S_BANDWIDTH_7MHZ );
		break;
		
	case BANDWIDTH_8_MHZ:
	default:
		p_mxl5005s_extra->SetBandwidthHz( dib, pTuner_Mxl5005 , MXL5005S_BANDWIDTH_8MHZ );	   
		break;
	}	
	pTuner_Mxl5005->SetRfFreqHz( dib, pTuner_Mxl5005 , fep->frequency);
	pTuner_Mxl5005->GetRfFreqHz( dib, pTuner_Mxl5005 , &actualfreqHz);
//	printk(" %s : MXL tuner frequency = %dHz\n", __FUNCTION__,actualfreqHz);    

	return 0;
}




static int rtdusb_tuner_quirk(struct dvb_frontend *fe)
{
	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	struct dvb_usb_device*	dib = p_state->dib;
	u8					R_Data = 0;

	
	//2 check  MT2060
	p_state->tuner_type = RTDUSB_TUNER_MT2060;
	p_state->tuner_addr = MT2060_BASE_ADDRESS;
	
	TUNER_RI2C( dib, MT2060_BASE_ADDRESS, MT2060_OFFSET, &R_Data , 1 );	
	if((R_Data&0xf0) == MT2060_CHECK_VAL)
	{
		p_state->pTuner = kzalloc( sizeof(TUNER_MT2060) , GFP_KERNEL );
		if(p_state->pTuner==NULL)
		{
			printk("allocate MT tuner Memory failed\n");
			kfree(p_state->pTuner);
			return 1;
		}			
				
		mt2060_tuner_struct_init(fe);
		printk("This device has the MT2060 onboard.....\n");
		return 0;
	}
		
	//2 MXL tuner (Default)		

	p_state->tuner_type = RTDUSB_TUNER_MXL5005;
	p_state->tuner_addr = MXL5005_BASE_ADDRESS;
	
	p_state->pTuner = kzalloc( sizeof(TUNER_MODULE) , GFP_KERNEL );	
	if(p_state->pTuner==NULL)
	{
		printk("allocate MXL tuner Memory failed\n");
		kfree(p_state->pTuner);
		return 1;
	}			
	p_state->pMxl5005sExtraModuleMemory = kzalloc( sizeof(MXL5005S_EXTRA_MODULE ) , GFP_KERNEL );	
	if(p_state->pMxl5005sExtraModuleMemory==NULL)
	{
		printk("allocate MXL tuner Extra Module Memory failed\n");
		kfree(p_state->pMxl5005sExtraModuleMemory);
		return 1;
	}			
		
		
	BuildMxl5005sModule((TUNER_MODULE **)&p_state->pTuner,
						 (TUNER_MODULE * )p_state->pTuner,
						 (MXL5005S_EXTRA_MODULE*)p_state->pMxl5005sExtraModuleMemory,
						 &p_state->BaseInterfaceModuleMemory,
						 &p_state->I2cBridgeModuleMemory,		//not needed
						 MXL5005_BASE_ADDRESS,			// 8-bit(0xc6);7-bit(0x63)
						 MXL5005S_STANDARD_DVBT);
		

	printk("This device has the MXL5005 onboard.....(Default)\n");		
	return 0;
	
}





int rtdusb_general_tuner_init(
	struct dvb_frontend*	fe)
{
	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	/* check which tuner is mounted on this device, in case this is unsure */
	rtdusb_tuner_quirk(fe);		

	switch (p_state->tuner_type) {
		
	case RTDUSB_TUNER_MXL5005: //add by chialing 070411
		return maxlinear_mxl5005_init(fe);

	case RTDUSB_TUNER_MT2060: //add by chialing 070523
		return microtune_mt2060_init(fe);
			
	default:
		break;
	}

		return 0;

}





int rtdusb_general_tuner_set(
	struct dvb_frontend*				fe,
	struct dvb_frontend_parameters*	fep)
{
	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	switch (p_state->tuner_type) {
		
		case RTDUSB_TUNER_MXL5005: //add by chialing 070411
			return maxlinear_mxl5005_set(fe, fep);
			
		case RTDUSB_TUNER_MT2060: //add by chialing 070523	
			return microtune_mt2060_set(fe , fep);
		default:
			warn("no pll programming routine found for tuner %d.\n",p_state->tuner_type);
			break;
	}
	return 0;
}




int rtdusb_general_tuner_release(
	struct dvb_frontend*	fe)
{
	struct rtd2830_state*	p_state = (struct rtd2830_state*)fe->demodulator_priv;
	switch (p_state->tuner_type){
		case RTDUSB_TUNER_MXL5005: 
			kfree(p_state->pTuner);
			kfree(p_state->pMxl5005sExtraModuleMemory); 
			break;		//dumb error 070608
		case RTDUSB_TUNER_MT2060:
			kfree(p_state->pTuner);
			break;		//dumb error 070608
		default:
			break;
	}
	return 0;	
}

