
#include "tuner_demod_io.h"


int rtd2831_usb_register_read(struct dvb_usb_device *dib,unsigned short offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;
 
        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_rcvctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_IN,										/* USB message request type value */
                (USB_BASE_ADDRESS<<8) + offset,						/* USB message value */
                0x0100,													/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */
 	
        if (ret != bytelength)
	{
		printk("error!! %s: ( offset , data ) = ( 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__, offset, *data , ret);
		return -EFAULT;
       }

	return ret; 
}

int rtd2831_usb_register_write(struct dvb_usb_device *dib,unsigned short offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;
 
        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_sndctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_OUT,										/* USB message request type value */
                (USB_BASE_ADDRESS<<8) + offset,						/* USB message value */
                0x0110,													/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */

        if (ret != bytelength)
	{
		printk("error!! %s: ( offset , data ) = ( 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__, offset, *data , ret);
		return -EFAULT;
       }

	return ret;
 }




int rtd2831_sys_register_read(struct dvb_usb_device *dib,unsigned short offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;
 
        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_rcvctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_IN,										/* USB message request type value */
                (SYS_BASE_ADDRESS<<8) + offset,						/* USB message value */
                0x0200,													/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */
			
        if (ret != bytelength)
	{
		printk("error!! %s: ( offset , data ) = ( 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__, offset, *data , ret);
		return -EFAULT;
       }

	return ret; 

  }
int rtd2831_sys_register_write(struct dvb_usb_device *dib,unsigned short offset, unsigned char *data, unsigned short bytelength)
{ 
	int ret = -ENOMEM;


        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_sndctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_OUT,										/* USB message request type value */
                (SYS_BASE_ADDRESS<<8) + offset,						/* USB message value */
                0x0210,													/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */
		
        if (ret != bytelength)
	{
		printk("error!! %s: ( offset , data ) = ( 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__, offset, *data , ret);
		return -EFAULT;
        }

	return ret;
 }

int rtd2831_demod_register_read(struct dvb_usb_device *dib,unsigned char page, unsigned char offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;

        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_rcvctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_IN,										/* USB message request type value */
                DEMOD_I2C_ADDRESS + (offset<<8),						/* USB message value */
                (0x0000 + page),										/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */
	
        if (ret != bytelength)
	{
		printk("error!! %s: ( page , offset , data ) = ( 0x%x , 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__, page,offset,*data,ret);
		return -EFAULT;
       }

	return ret;  
}

int rtd2831_demod_register_write(struct dvb_usb_device *dib,unsigned char page, unsigned char offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;

        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_sndctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_OUT,										/* USB message request type value */
                DEMOD_I2C_ADDRESS + (offset<<8),						/* USB message value */
                (0x0010 + page),										/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */

        if (ret != bytelength)
	{
		printk("error!! %s: ( page , offset , data ) = ( 0x%x , 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__, page,offset,*data,ret);
		return -EFAULT;
        }

	return ret;
 }






int
RTD2831_READ_BYTES(
struct dvb_usb_device*	dib,
    int			page_addr,
    int			byte_addr,
    unsigned char*	buf,
    unsigned short	byte_num)
{
	int ret = -ENOMEM;

	if( byte_num != 1 && byte_num !=2 && byte_num !=4)
	{
		printk("error!! %s: length = %d \n", __FUNCTION__, byte_num);
		return 1;
	}

	if ((ret = mutex_lock_interruptible(&dib->usb_mutex)))
		return ret;
	
	if( page_addr == RTD2831U_USB )
	{
		rtd2831_usb_register_read( dib , byte_addr , buf , byte_num);
	}
	else if ( page_addr == RTD2831U_SYS )
	{
		rtd2831_sys_register_read( dib , byte_addr , buf , byte_num);
	}
	else
	{
		rtd2831_demod_register_read( dib , page_addr , byte_addr , buf , byte_num);
	}	

	mutex_unlock(&dib->usb_mutex);

	return 0;
	
}



int
RTD2831_WRITE_BYTES(
struct dvb_usb_device*	dib,
    int			page_addr,
    int			byte_addr,
    unsigned char*	buf,
    unsigned short	byte_num)
{
	int ret = -ENOMEM;

	if( byte_num != 1 && byte_num !=2 && byte_num !=4)
	{
		printk("error!! %s: length = %d \n", __FUNCTION__, byte_num);
		return 1;
	}

	if ((ret = mutex_lock_interruptible(&dib->usb_mutex)))
		return ret;

	
	if( page_addr == RTD2831U_USB )
	{
		rtd2831_usb_register_write( dib , byte_addr , buf , byte_num);
	}
	else if ( page_addr == RTD2831U_SYS )
	{
		rtd2831_sys_register_write( dib , byte_addr , buf , byte_num);
	}
	else
	{
		rtd2831_demod_register_write( dib , page_addr , byte_addr , buf , byte_num);
	}	

	mutex_unlock(&dib->usb_mutex);

	return 0;
	
}

/////////////////////////////////////////tuner register read/write///////////////////////////////////////

int rtd2831_tuner_register_read(struct dvb_usb_device *dib,unsigned char device_address, unsigned char offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;
 
        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_rcvctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_IN,										/* USB message request type value */
                device_address+(offset<<8),							/* USB message value */
                0x0300,													/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */
			
        if (ret != bytelength)
 	{
		printk("error!! %s: ( offset , data ) = ( 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__,offset,*data,ret);
		return -EFAULT;
        }

	return ret;   
 
}

int rtd2831_tuner_register_write(struct dvb_usb_device *dib,unsigned char device_address, unsigned char offset, unsigned char *data, unsigned short bytelength)
{
	int ret = -ENOMEM;
 
        ret = usb_control_msg(dib->udev,								/* pointer to device */
                usb_sndctrlpipe( dib->udev,RTD2831_CTRL_ENDPOINT),		/* pipe to control endpoint */
                0,														/* USB message request value */
                SKEL_VENDOR_OUT,										/* USB message request type value */
                device_address+(offset<<8),							/* USB message value */
                0x0310,													/* USB message index value */
                data,													/* pointer to the receive buffer */
                bytelength,												/* length of the buffer */
                DIBUSB_I2C_TIMEOUT);									/* time to wait for the message to complete before timing out */
			
        if (ret != bytelength)
	{
		printk("error!! %s: ( offset , data ) = ( 0x%x , 0x%x ) size != %d bytes \n", __FUNCTION__,offset,*data,ret);
		return -EFAULT;
       }

	return ret;
 }



int
TUNER_RI2C(
struct dvb_usb_device *dib,
    u8				tuner_addr,    
    u8				reg_addr,
    unsigned char*		buf,
    unsigned short		byte_num)
{
	unsigned char			i2crepeat;
	int ret = -ENOMEM;

	if( byte_num != 1 && byte_num !=2 && byte_num !=4)
	{
		printk("error!! %s: length = %d \n", __FUNCTION__, byte_num);
		return 1;
	}

	if ((ret = mutex_lock_interruptible(&dib->usb_mutex)))
		return ret;

	
       rtd2831_demod_register_read(dib,1,1,&i2crepeat,1);
       i2crepeat |= 8;
       rtd2831_demod_register_write(dib,1,1,&i2crepeat,1);

	rtd2831_tuner_register_read( dib , tuner_addr , reg_addr , buf , byte_num);

	mutex_unlock(&dib->usb_mutex);

	return 0;
}



int
TUNER_WI2C(
struct dvb_usb_device *dib,
    u8				tuner_addr,    
    u8				reg_addr,
    unsigned char*		buf,
    unsigned short		byte_num)
{

	unsigned char			i2crepeat;
	int ret = -ENOMEM;

	if( byte_num != 1 && byte_num !=2 && byte_num !=4)
	{
		printk("error!! %s: length = %d \n", __FUNCTION__, byte_num);
		return 1;
	}
	
	if ((ret = mutex_lock_interruptible(&dib->usb_mutex)))
		return ret;

  
       rtd2831_demod_register_read(dib,1,1,&i2crepeat,1);
       i2crepeat |= 8;
       rtd2831_demod_register_write(dib,1,1,&i2crepeat,1);

	rtd2831_tuner_register_write( dib , tuner_addr , reg_addr , buf , byte_num );

	mutex_unlock(&dib->usb_mutex);
	
	return 0;
}




